package de.meinefirma.meinprojekt.backingbeans;

import javax.management.MBeanServerConnection;
import java.lang.management.*;
import java.text.SimpleDateFormat;
import java.util.*;

public enum CpuGcMemTx
{
   INSTANCE;

   private static final long MILLISEK500 = 500;
   private final  SimpleDateFormat             yyyyMMddHHmmss = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss" );
   private final  MBeanServerConnection        mbs            = ManagementFactory.getPlatformMBeanServer();
   private final  RuntimeMXBean                rtMXBean       = ManagementFactory.getRuntimeMXBean();
   private final  OperatingSystemMXBean        opSystMXBean   = ManagementFactory.getOperatingSystemMXBean();
   private final  List<GarbageCollectorMXBean> gcMXBeans      = CpuGcMemTxMBeanUtil.getGarbageCollectorMXBeans( mbs );
   private long   lastUpTimeMs;
   private long   lastGcTimeMs;
   private long   lastCpuTimeNs;
   private long[] lastTxCounts = new long[2];
   private String resultCpuGcMem;
   private String resultTx;

   public final void run()
   {
      if( mbs == null || rtMXBean == null || gcMXBeans == null ) { return; }
      long cpuCount = Math.max( 1, ( opSystMXBean != null ) ? opSystMXBean.getAvailableProcessors() : 1 );
      long upTimeMs = rtMXBean.getUptime();
      if( upTimeMs - lastUpTimeMs < MILLISEK500 ) { return; }
      long     gcTimeMs       = CpuGcMemTxMBeanUtil.readGarbageCollectionTimeMs( gcMXBeans );
      double   gcTimePercent  = CpuGcMemTxMBeanUtil.calculateGarbageCollectionTimePercent( gcTimeMs, lastGcTimeMs, upTimeMs, lastUpTimeMs, cpuCount );
      long     cpuTimeNs      = CpuGcMemTxMBeanUtil.readCpuTimeNs( mbs );
      int      cpuTimePercent = CpuGcMemTxMBeanUtil.calculateCpuTimePercent( cpuTimeNs, lastCpuTimeNs, upTimeMs, lastUpTimeMs, cpuCount );
      long[]   txCounts       = CpuGcMemTxMBeanUtil.readTxCountsWebLogic( mbs, "AdminServer" );
      String[] txPerSec       = CpuGcMemTxMBeanUtil.calculatePerSecond( txCounts, lastTxCounts, upTimeMs, lastUpTimeMs );

      StringBuilder cpuGcMem = new StringBuilder();
      cpuGcMem.append( yyyyMMddHHmmss.format( new Date() ) ).append( "; CPUs: " ).append( cpuCount );
      System.gc();
      long memTotal = Runtime.getRuntime().totalMemory();
      long memFree  = Runtime.getRuntime().freeMemory();
      cpuGcMem.append( "; CPU: "  ).append( cpuTimePercent );
      cpuGcMem.append( " %; GC: " ).append( CpuGcMemTxMBeanUtil.doubleString1( gcTimePercent ) );
      cpuGcMem.append( " %; MemUsed: "   ).append( CpuGcMemTxMBeanUtil.doubleString1( (memTotal - memFree) / (1024. * 1024.) ) );
      cpuGcMem.append( " MB; MemTotal: " ).append( CpuGcMemTxMBeanUtil.doubleString1(  memTotal / (1024. * 1024.) ) ).append( " MB" );
      resultCpuGcMem = cpuGcMem.toString();

      resultTx = null;
      if( txCounts != null && txCounts.length > 1 && (txCounts[0] > 0 || txCounts[1] > 0) ) {
         StringBuilder tx = new StringBuilder();
         tx.append( yyyyMMddHHmmss.format( new Date() ) );
         tx.append( "; Tx/Sec: " ).append( txPerSec[0] ).append( "; TxRollback/Sec: " ).append( txPerSec[1] );
         tx.append( "; TxSum: "  ).append( txCounts[0] ).append( "; TxRollbackSum: " ).append( txCounts[1] );
         lastTxCounts[0] = txCounts[0];
         lastTxCounts[1] = txCounts[1];
         resultTx = tx.toString();
      }
      lastUpTimeMs  = upTimeMs;
      lastCpuTimeNs = cpuTimeNs;
      lastGcTimeMs  = gcTimeMs;
   }

   public final String getCpuGcMem()
   {
      return resultCpuGcMem;
   }

   public final String getTx()
   {
      return resultTx;
   }
}
