package de.meinefirma.meinprojekt.backingbeans;

import java.io.InputStream;
import java.net.InetAddress;
import java.util.*;
import javax.faces.bean.*;
import javax.faces.context.*;
import javax.persistence.*;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

@ManagedBean
@ViewScoped
public class InspectImplementationsVersionsBean
{
   // Falls es keinen EntityManager gibt, muessen die folgenden zwei Zeilen deaktiviert werden,
   // ansonsten muss der PERSISTENCE_UNIT_NAME angepasst werden:
   private static final String PERSISTENCE_UNIT_NAME = "MeineJpaPU";
   private EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_UNIT_NAME );
   private static final int MAX_LEN = 160;
   private static final String KEY_IN_SESSION_INSPECTLIBSANDCLASSES = "InspectLibsAndClasses";
   private InspectLibsAndClasses inspectLibsAndClasses;

   public InspectImplementationsVersionsBean()
   {
      final ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
      final ServletContext  servletContext  = (ServletContext) externalContext.getContext();
      Map<String,Object> sessionMap = externalContext.getSessionMap();
      inspectLibsAndClasses = (InspectLibsAndClasses) sessionMap.get( KEY_IN_SESSION_INSPECTLIBSANDCLASSES );
      sessionMap.put( KEY_IN_SESSION_INSPECTLIBSANDCLASSES, null );
      if( inspectLibsAndClasses == null ) {
         inspectLibsAndClasses = new InspectLibsAndClasses(
            new InspectLibsAndClasses.ServletContextStellvertreter() {
               @Override public InputStream getResourceAsStream( String resourcePath ) {
                  return servletContext.getResourceAsStream( resourcePath );
               }
               @Override public Set<String> getResourcePaths( String resourceLibPath ) {
                  return servletContext.getResourcePaths( resourceLibPath );
               }
            } );
      }
   }

   // Bitte beachten: Je nach eingebundenen Libs stehen nicht alle Funktionen zur Verfuegung
   public List<String[]> getImplementationsVersionsList() throws Exception
   {
      FacesContext ctx = FacesContext.getCurrentInstance();
      HttpServletRequest request = (HttpServletRequest) ctx.getExternalContext().getRequest();
      String angefragterServer = request.getServerName();
      if( request.getServerPort() != request.getLocalPort() ) { angefragterServer += ":" + request.getServerPort(); }
      angefragterServer += " (";
      if( request.getLocalName() != null && !request.getLocalName().equals( request.getServerName() )
                                         && !request.getLocalName().equals( request.getLocalAddr() )) {
         angefragterServer += request.getLocalName() + ", ";
      }
      angefragterServer += request.getLocalAddr() + ":" + request.getLocalPort() + ")";

      String requestClientServer = "Anfragender Client: " + request.getRemoteUser() + " (" + request.getRemoteAddr() + ":" + request.getRemotePort() +
                                   "), angefragter Server: " + angefragterServer;
      String betriebssystemPatchlevel = System.getProperty( "sun.os.patch.level" );
      betriebssystemPatchlevel =
            ( betriebssystemPatchlevel == null || betriebssystemPatchlevel.trim().isEmpty() || "unknown".equalsIgnoreCase( betriebssystemPatchlevel ) )
            ? "" : (betriebssystemPatchlevel + ", " );
      List<String[]> result = new ArrayList<>();
      addKeyValue( "Host-Name", InetAddress.getLocalHost().getHostName() + " (" + InetAddress.getLocalHost().getHostAddress() + ")", result );
      addKeyValue( "Betriebssystem", System.getProperty( "os.name" ) + ", " + System.getProperty( "os.version" ) + ", "
            + betriebssystemPatchlevel + System.getProperty( "os.arch" ) + ", "
            + System.getProperty( "sun.arch.data.model" ) + " bit", result );
      addKeyValue( "Java-Version", System.getProperty( "java.runtime.version" ) + " (" + System.getProperty( "java.vm.version" ) + ")", result );
      addKeyValue( "Server-Info", ((ServletContext) ctx.getExternalContext().getContext()).getServerInfo(), result );
      addKeyValue( "Context-Name", ctx.getExternalContext().getContextName(), result );
      addKeyValue( "DatatypeFactory", javax.xml.datatype.DatatypeFactory.DATATYPEFACTORY_PROPERTY, result );
      addKeyValue( "DatatypeFactoryImpl", javax.xml.datatype.DatatypeFactory.DATATYPEFACTORY_IMPLEMENTATION_CLASS, result );
      addClassName( "XMLGregorianCalendar", javax.xml.datatype.DatatypeFactory.newInstance().newXMLGregorianCalendar( "2015" ).getClass(), result );
      try {
         addClassName( "JAXB-Provider", javax.xml.bind.JAXBContext.newInstance().getClass(), result );
      } catch( NoClassDefFoundError e ) { /* ok */ }
      try {
         Class<?> bcClss = Class.forName( "org.bouncycastle.bcpg.BCPGKey" );
         addImplementationVersionFromPackage( "BouncyCastle", bcClss.getPackage(), result );
      } catch( ClassNotFoundException e ) { /* ok */ }
      addImplementationVersionFromPackage( "Slf4j", "org.slf4j", result );
      addSpecificationVersionFromPackage( "EJB-Spec", "javax.ejb", result );
      addImplementationVersionFromPackage( "Hibernate-Validator", "org.hibernate.validator", result );
      try {
         Class<?> oraClss = Class.forName( "oracle.jdbc.OracleDriver" );
         addSpecificationVersionFromPackage( "Oracle-JDBC-Spec", oraClss.getPackage(), result );
         addImplementationVersionFromPackage( "Oracle-JDBC-Impl", oraClss.getPackage(), result );
      } catch( ClassNotFoundException e ) { /* ok */ }
      addSpecificationVersionFromPackage( "JPA-Spec", EntityManager.class.getPackage(), result );
      addImplementationVersionFromPackage( "JPA-Impl", EntityManager.class.getPackage(), result );
      if( emf != null ) {
         addImplementationVersionFromPackage( "JPA-Provider", emf.getClass().getPackage(), result );
         addKeyValue( "JPA-EntityManager", emf.createEntityManager().getDelegate().getClass().getName(), result );
      }
      addImplementationVersionFromPackage( "EclipseLink", "org.eclipse.persistence", result );
      addImplementationVersionFromPackage( "Jetty", "org.eclipse.jetty.server", result );
      addSpecificationVersionFromPackage( "Servlet-Spec", "javax.servlet", result );
      addImplementationVersionFromPackage( "JAX-RS", "javax.ws.rs", result );
      addImplementationVersionFromPackage( "Jersey 1", "com.sun.jersey.api.client", result );
      addImplementationVersionFromPackage( "Jersey 2", "org.glassfish.jersey.servlet", result );
      addSpecificationVersionFromPackage( "JSF-Spec", FacesContext.class.getPackage(), result );
      addImplementationVersionFromPackage( "JSF-API", FacesContext.class.getPackage(), result );
      addImplementationVersionFromPackage( "JSF-Impl", ctx.getClass().getPackage(), result );
      addImplementationVersionFromPackage( "MyFaces", "org.apache.myfaces.context", result );
      addClassName( "JSF-RenderKit", ctx.getRenderKit().getClass(), result );
      addImplementationVersionFromPackage( "Tomahawk", "org.apache.myfaces.tomahawk.application", result );
      addImplementationVersionFromPackage( "PrimeFaces", "org.primefaces.context", result );
      addKeyValue( "Context", ctx.getExternalContext().getContext().toString().replace( "{", " {" ).replace( ",", ", " ), result );
      addKeyValue( "Request-Client/Server", requestClientServer, result );
      addKeyValue( "Request-Objekt", ctx.getExternalContext().getRequest(), result );
      addKeyValue( "RequestParameterMap", ctx.getExternalContext().getRequestParameterMap(), result );
      String locMsg = null;
      locMsg = InspectLibsAndClasses.concatWithComma( locMsg, "ViewRoot-Locale=", ctx.getViewRoot().getLocale() );
      locMsg = InspectLibsAndClasses.concatWithComma( locMsg, "Default-Locale=",  ctx.getApplication().getDefaultLocale() );
      locMsg = InspectLibsAndClasses.concatWithComma( locMsg, "MessageBundle=",   ctx.getApplication().getMessageBundle() );
      addKeyValue( "Locale/MessageBundle", locMsg, result );
      // Die folgenden 7 Zeilen nur, falls "CpuGcMemTx" eingebunden wird:
      CpuGcMemTx.INSTANCE.run();
      if( CpuGcMemTx.INSTANCE.getCpuGcMem() != null ) {
         addKeyValue( "Cpu, Gc, Mem", CpuGcMemTx.INSTANCE.getCpuGcMem(), result );
      }
      if( CpuGcMemTx.INSTANCE.getTx() != null ) {
         addKeyValue( "Transaktionen", CpuGcMemTx.INSTANCE.getTx(), result );
      }
      return result;
   }

   public List<String[]> getSessionMapList()
   {
      List<String[]> result = new ArrayList<>();
      Map<String,Object> sessionMap = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
      Map<String,Object> sessionMapSorted = new TreeMap<>( sessionMap );
      for( Map.Entry<String,Object> entry : sessionMapSorted.entrySet() ) {
         addKeyValue( entry.getKey(), entry.getValue(), MAX_LEN / 2, result );
      }
      return result;
   }

   public List<String> getClasspathString()
   {
      String classpath = System.getProperty( "java.class.path" );
      if( classpath == null ) { return null; }
      return Arrays.asList( new String[] { classpath.replace( ";", "; " ).replace( ":", ": " ) } );
   }

   public List<String[]> getLibsAndMulClassCnt()
   {
      return inspectLibsAndClasses.getLibsAndMulClassCnt();
   }

   public String zeigeMultipleClasses()
   {
      Map<String,Object> sessionMap = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
      sessionMap.put( KEY_IN_SESSION_INSPECTLIBSANDCLASSES, inspectLibsAndClasses );
      return "multipleClasses.xhtml";
   }

   public List<String[]> getMultipleClassesList()
   {
      return inspectLibsAndClasses.getMultipleClassesList();
   }

   public String zuInspectImplementationsVersions()
   {
      Map<String,Object> sessionMap = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
      sessionMap.put( KEY_IN_SESSION_INSPECTLIBSANDCLASSES, inspectLibsAndClasses );
      return "inspectImplementationsVersions.xhtml";
   }

   // Diese Methode funktioniert nur in bestimmten Konstellationen und nur, wenn ein Zugriff auf
   // EntityManagerFactory und EntityManagerFactory.getProperties() moeglich ist:
   public List<String[]> getEntityManagerPropertiesList()
   {
      List<String[]> result = new ArrayList<>();
      try {
         if( emf == null || emf.getProperties() == null ) { return null; }
         addImplementationVersionFromPackage( "JPA-EntityManager", EntityManager.class.getPackage(), result );
         addImplementationVersionFromPackage( "JPA-Provider", emf.getClass().getPackage(), result );
         for( Map.Entry<String,Object> entry : emf.getProperties().entrySet() ) {
            addKeyValue( entry.getKey(), entry.getValue(), MAX_LEN / 2, result );
         }
      } catch( Exception ex ) {
         addKeyValue( ex.getClass().toString(), ex.getMessage(), result );
      }
      return result;
   }

   private static void addSpecificationVersionFromPackage( String key, String pckg, List<String[]> result )
   {
      addSpecificationVersionFromPackage( key, Package.getPackage( pckg ), result );
   }

   private static void addSpecificationVersionFromPackage( String key, Package pckg, List<String[]> result )
   {
      if( pckg == null ) { return; }
      String val  = pckg.getSpecificationTitle();
      if( val == null ) { val = pckg.getName(); }
      String vers = pckg.getSpecificationVersion();
      if( vers != null && !val.contains( vers ) ) { val += " "  + pckg.getSpecificationVersion(); }
      if( pckg.getSpecificationVendor() != null ) { val += ", " + pckg.getSpecificationVendor(); }
      addKeyValue( key, val, result );
   }

   private static void addImplementationVersionFromPackage( String key, String pckg, List<String[]> result )
   {
      addImplementationVersionFromPackage( key, Package.getPackage( pckg ), result );
   }

   private static void addImplementationVersionFromPackage( String key, Package pckg, List<String[]> result )
   {
      if( pckg == null ) { return; }
      String val = pckg.getImplementationTitle();
      if( val == null ) { val = pckg.getName(); }
      if( pckg.getImplementationVersion() != null ) { val += " "  + pckg.getImplementationVersion(); }
      if( pckg.getImplementationVendor()  != null ) { val += ", " + pckg.getImplementationVendor(); }
      addKeyValue( key, val, result );
   }

   private static void addClassName( String key, Class<?> clss, List<String[]> result )
   {
      if( clss == null ) { return; }
      String val = clss.toString();
      if( val.startsWith( "class " ) ) { val = val.substring( 6 ); }
      addKeyValue( key, val, result );
   }

   private static void addKeyValue( String key, Object value, List<String[]> result )
   {
      addKeyValue( key, value, MAX_LEN, result );
   }

   private static void addKeyValue( String key, Object value, int maxlen, List<String[]> result )
   {
      if( key == null || value == null ) { return; }
      String val = value.toString();
      if( val == null || val.trim().length() == 0 || "{}".equals( val ) ) { return; }
      int maxlen2 = maxlen/2 - 6;
      String key2 = ( key.length() <= maxlen ) ? key : (key.substring( 0, maxlen2 ) + "... ... ..." + key.substring( key.length() - maxlen2 ) );
      String val2 = ( val.length() <= maxlen ) ? val : (val.substring( 0, maxlen2 ) + "... ... ..." + val.substring( val.length() - maxlen2 ) );
      result.add( new String[] { key2, val2 } );
   }

   public void setAusgewaehlteLib( String ausgewaehlteLib )
   {
      inspectLibsAndClasses.setAusgewaehlteLib( ausgewaehlteLib );
   }
}
