package de.meinefirma.meinprojekt.backingbeans;

import java.util.*;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import de.meinefirma.meinprojekt.entities.*;

// Backing Bean, verknuepft die Daten und die Geschaeftslogik mit der JSF-View
@ManagedBean
@ViewScoped
public class MeineBackingBean
{
   private static final String KEY_IN_SESSION = "einzelnesElement";
   private List<MeineEntity> meineDatenListe;
   private MeineEntity einzelnesElement;
   private Map<MeineEntity,Boolean> merkerMap = new HashMap<>();

   public MeineBackingBean()
   {
      meineDatenListe = MeineEntityDao.findAll();

      einzelnesElement = (MeineEntity) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get( KEY_IN_SESSION );
      FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put( KEY_IN_SESSION, null );

      // Die folgenden Zeilen erzeugen eine Daten-Vorbelegung fuer erste einfache Tests:
      if( meineDatenListe == null ) {
         meineDatenListe = MeineEntityDao.createTestDaten();
      }
   }

   public String neuesElementErstellen()
   {
      meineDatenListe = MeineEntityDao.findAll();
      int idNeu = 0;
      for( MeineEntity me : meineDatenListe ) {
         idNeu = Math.max( idNeu, me.getId().intValue() + 1 );
      }
      einzelnesElement = new MeineEntity( idNeu, null, new Date() );
      FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put( KEY_IN_SESSION, einzelnesElement );
      return "einzelnesElementEditieren.xhtml";
   }

   public String einzelnesElementEditieren()
   {
      FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put( KEY_IN_SESSION, einzelnesElement );
      return "einzelnesElementEditieren.xhtml";
   }

   public String einzelnesElementSpeichern()
   {
      MeineEntityDao.saveOrUpdateEntity( einzelnesElement );
      return "tabelle.xhtml";
   }

   public String markierteElementeLoeschen()
   {
      meineDatenListe = MeineEntityDao.findAll();
      for( Map.Entry<MeineEntity,Boolean> mapEntry : merkerMap.entrySet() ) {
         if( mapEntry.getValue().booleanValue() && !meineDatenListe.remove( mapEntry.getKey() ) ) {
            addFacesMessage( FacesMessage.SEVERITY_ERROR, "Fehler: Datenelement existiert nicht mehr." );
         }
      }
      MeineEntityDao.writeAll( meineDatenListe );
      merkerMap.clear();
      return null;
   }

   public void addFacesMessage( FacesMessage.Severity severity, String msg )
   {
      FacesMessage facesMsg = new FacesMessage( severity, msg, null );
      FacesContext.getCurrentInstance().addMessage( null, facesMsg );
   }

   public List<MeineEntity>        getMeineDatenListe()  { return meineDatenListe; }
   public MeineEntity              getEinzelnesElement() { return einzelnesElement; }
   public Map<MeineEntity,Boolean> getMerkerMap()        { return merkerMap; }
   public void setMeineDatenListe(  List<MeineEntity> meineDatenListe )  { this.meineDatenListe = meineDatenListe; }
   public void setEinzelnesElement( MeineEntity einzelnesElement )       { this.einzelnesElement = einzelnesElement; }
   public void setMerkerMap(        Map<MeineEntity,Boolean> merkerMap ) { this.merkerMap = merkerMap; }
}
