package de.meinefirma.meinprojekt.entities;

import java.io.*;
import java.util.*;

/** DAO fuer leichteren Zugriff auf die gespeicherten Datenobjekte */
public class MeineEntityDao
{
   private static final String DATEI = "MeineDaten.dat";

   @SuppressWarnings("unchecked")
   public static List<MeineEntity> findAll()
   {
      return readObject( List.class, DATEI );
   }

   public static void writeAll( List<MeineEntity> entitiesList )
   {
      writeObject( entitiesList, DATEI );
   }

   public static void saveOrUpdateEntity( MeineEntity entity )
   {
      List<MeineEntity> entitiesList = findAll();
      if( entitiesList == null || entity == null ) { return; }
      entitiesList.remove( entity );
      entitiesList.add( entity );
      writeAll( entitiesList );
   }

   public static List<MeineEntity> createTestDaten()
   {
      List<MeineEntity> entitiesList = new ArrayList<>();
      entitiesList.add( new MeineEntity(  1, "Apfelsinen", new Date() ) );
      entitiesList.add( new MeineEntity(  7, "Birnen",     new Date( (new Date()).getTime() - 1000000000 ) ) );
      entitiesList.add( new MeineEntity( 42, "Tomaten",    new Date( (new Date()).getTime() - 2000000000 ) ) );
      writeAll( entitiesList );
      return entitiesList;
   }

   // Primitiver Persistenzmechanismus zum Speichern der Daten (fuer Tests nur in Datei, statt DB per JPA):

   public static <T> T readObject( Class<T> clss, String datei )
   {
      if( !(new File( datei )).exists() ) { return null; }
      try( ObjectInputStream in = new ObjectInputStream( new FileInputStream( datei ) ); ) {
         return clss.cast( in.readObject() );
      } catch( ClassNotFoundException | IOException e ) {
         throw new RuntimeException( e );
      }
   }

   public static void writeObject( Object obj, String datei )
   {
      try( ObjectOutputStream out = new ObjectOutputStream( new FileOutputStream( datei ) ); ) {
        out.writeObject( obj );
        out.flush();
      } catch( IOException e ) {
         throw new RuntimeException( e );
      }
   }
}
