package de.meinefirma.meinprojekt.pageddata;

import java.util.*;
import javax.faces.event.ActionEvent;
import org.apache.myfaces.custom.datascroller.ScrollerActionEvent;

public class PagedDataList<D> implements List<D>
{
   private PagedDataReader<D> dao;
   private List<D> onePageOfData;
   private int     firstRowIndex = 0;
   private int     rowsPerPage;
   private int     rowsCountAll;
   private String  sortColumn;
   private boolean sortAscending = true;

   /** Konstruktor, dao = Data Acces Object, rowsPerPage = Anzahl in JSF-Tabelle angezeigter Zeilen */
   public PagedDataList( PagedDataReader<D> dao, int rowsPerPage )
   {
      this.dao         = dao;
      this.rowsPerPage = rowsPerPage;
      refreshPageOfData();
   }

   /** Einzelnes Datenelement (= Zeile), der Index bezieht sich auf die Gesamtgroesse */
   @Override
   public D get( int i )
   {
      int j = i - firstRowIndex;
      if( j < 0 || j >= onePageOfData.size() ) { return null; }
      return onePageOfData.get( j );
   }

   /** Gesamtgroesse der Datenliste (nicht die Groesse der aktuellen Seite) */
   @Override
   public int size()
   {
      return rowsCountAll;
   }

   @Override
   public boolean isEmpty()
   {
      return rowsCountAll == 0;
   }

   public void scrollerAction( ActionEvent event )
   {
      if( event instanceof ScrollerActionEvent ) {
         String facet = ((ScrollerActionEvent) event).getScrollerfacet();
         int    page  = ((ScrollerActionEvent) event).getPageIndex();
         int    rowsCountLastPage = rowsCountAll % rowsPerPage;
         if( rowsCountLastPage == 0 ) { rowsCountLastPage = rowsPerPage; }
         if( page == -1 ) {
            if( "next".equals( facet ) ) {
               firstRowIndex += rowsPerPage;
            } else if( "previous".equals( facet ) ) {
               firstRowIndex -= rowsPerPage;
            } else if( "fastf".equals( facet ) ) {
               firstRowIndex += 10 * rowsPerPage;
            } else if( "fastr".equals( facet ) ) {
               firstRowIndex -= 10 * rowsPerPage;
            } else if( "first".equals( facet ) ) {
               firstRowIndex = 0;
            } else if( "last".equals( facet ) ) {
               firstRowIndex = rowsCountAll - rowsCountLastPage;
            }
         } else {
            firstRowIndex = (page - 1) * rowsPerPage;
         }
         if( firstRowIndex < 0 ) { firstRowIndex = 0; }
         if( firstRowIndex >= rowsCountAll ) { firstRowIndex = rowsCountAll - rowsCountLastPage; }
         refreshPageOfData();
      }
   }

   public void setSortColumn( String sortColumn )
   {
      boolean diff = this.sortColumn == null || !this.sortColumn.equals( sortColumn );
      this.sortColumn = sortColumn;
      if( diff ) { refreshPageOfData(); }
   }

   public void setSortAscending( boolean sortAscending )
   {
      boolean diff = this.sortAscending != sortAscending;
      this.sortAscending = sortAscending;
      if( diff ) { refreshPageOfData(); }
   }

   public List<D> getSortableEntities() { return this; }
   public int     getFirstRowIndex()    { return firstRowIndex; }
   public int     getRowsPerPage()      { return rowsPerPage; }
   public String  getSortColumn()       { return sortColumn; }
   public boolean isSortAscending()     { return sortAscending; }

   private void refreshPageOfData()
   {
      onePageOfData = dao.readPageOfData( sortColumn, sortAscending, firstRowIndex, rowsPerPage );
      rowsCountAll  = dao.getRowsCountAll();
   }

   @Override public boolean add( D arg0 ) { throw new UnsupportedOperationException( "add" ); }
   @Override public void add( int arg0, D arg1 ) { throw new UnsupportedOperationException( "add" ); }
   @Override public boolean addAll( Collection<? extends D> arg0 ) { throw new UnsupportedOperationException( "addAll" ); }
   @Override public boolean addAll( int arg0, Collection<? extends D> arg1 ) { throw new UnsupportedOperationException( "addAll" ); }
   @Override public void clear() { throw new UnsupportedOperationException( "clear" ); }
   @Override public boolean contains( Object arg0 ) { throw new UnsupportedOperationException( "contains" ); }
   @Override public boolean containsAll( Collection<?> arg0 ) { throw new UnsupportedOperationException( "containsAll" ); }
   @Override public int indexOf( Object arg0 ) { throw new UnsupportedOperationException( "indexOf" ); }
   @Override public Iterator<D> iterator() { throw new UnsupportedOperationException( "iterator" ); }
   @Override public int lastIndexOf( Object arg0 ) { throw new UnsupportedOperationException( "lastIndexOf" ); }
   @Override public ListIterator<D> listIterator() { throw new UnsupportedOperationException( "listIterator" ); }
   @Override public ListIterator<D> listIterator( int arg0 ) { throw new UnsupportedOperationException( "listIterator" ); }
   @Override public boolean remove( Object arg0 ) { throw new UnsupportedOperationException( "remove" ); }
   @Override public D remove( int arg0 ) { throw new UnsupportedOperationException( "remove" ); }
   @Override public boolean removeAll( Collection<?> arg0 ) { throw new UnsupportedOperationException( "removeAll" ); }
   @Override public boolean retainAll( Collection<?> arg0 ) { throw new UnsupportedOperationException( "retainAll" ); }
   @Override public D set( int arg0, D arg1 ) { throw new UnsupportedOperationException( "set" ); }
   @Override public List<D> subList( int arg0, int arg1 ) { throw new UnsupportedOperationException( "subList" ); }
   @Override public Object[] toArray() { throw new UnsupportedOperationException( "toArray" ); }
   @Override public <T> T[] toArray( T[] arg0 ) { throw new UnsupportedOperationException( "toArray" ); }
}
